////////////////////////////////////////////////////////////////////////////////
// IMPORTANT: This file is not executed. It is only used for type checking and
// autocomplete when working in a text editor such as VS Code. In the JavaScript
// runtime where your theme is executed, the interfaces defined in this file are
// implemented using native objects.
////////////////////////////////////////////////////////////////////////////////

interface DefineBranchRuleFunction {
    /**
     * @param match - Outline path to match branch row
     * @param apply - Function to modify BranchStyle when rule matches
     */
    (match: string, apply: (environment: Environment, branch: BranchStyle) => void);
}

export declare var defineBranchRule: DefineBranchRuleFunction;

interface DefineRowRuleFunction {
    /**
     * @param match - Outline path to match row
     * @param apply - Function to modify RowStyle when rule matches
     */
    (match: string, apply: (environment: Environment, row: RowStyle) => void);
}

export declare var defineRowRule: DefineRowRuleFunction;

interface DefineRunRuleFunction {
    /**
     * @param match - Outline path to match run
     * @param apply - Function to modify RunStyle when rule matches
     */
    (match: string, apply: (environment: Environment, run: RunStyle) => void): Object;
}

export declare var defineRunRule: DefineRunRuleFunction;

/**
 * The environment passed into the style rule functions. Rule functions are
 * recomputed when the environment changes.
 */
export interface Environment {
    darkMode: boolean;
    accentColor: Color;
    foregroundColor: Color;
    backgroundColor: Color;
    viewportWidth: number;
    viewportHeight: number;
}

export interface BranchStyle {
    /** The inherited style for branch's rows */
    rows: RowStyle
    /** The branch background, */
    background: Background;
}

export interface RowStyle {
    /** The row font */
    font: Font;
    /** The row foreground color */
    color: Color;
    /** The vertical row spacing */
    spacing: number;
    /** The line height multiple */
    lineHeightMultiple: number;
    /** The row controls */
    controls: Controls;
    /** The row opacity */
    opacity: number;
    /** Content margins */
    contentMargins: Insets;
    /** Content background */
    contentBackground: Background;
}

/** The style for a run of text. */
export interface RunStyle {
    /** The run foreground color, inherited from row */
    color: Color;
    /** The run background color */
    backgroundColor: Color | undefined;
    /** The run font, inherited from row */
    font: Font;
    /** The run kerning (default 0) */
    kerning: number;
    /** The run tracking (default 0) */
    tracking: number;
    /** The run ligatures */
    ligatures: 'default' | 'none' | 'all';
    /** The run baseline offset */
    baselineOffset: number;
    /** The run text decorations */
    textDecorations: TextDecorations;
    /** The run margins */
    //TODOmargins: Insets;
    /** The run background */
    //TODO background: Background;
}

export interface Controls {
    // TODO
    //handleColor: Color;
    //guideColor: Color;
    //focusColor: Color;
}

/** Background - Background for a branch, row, or run. */
export interface Background {
    /** Background opacity (0-1) */
    opacity: number;
    /** Background color */
    color: Color;
    /** Border color */
    borderColor: Color;
    /** Border width */
    borderWidth: number;
    /** Corner radius */
    cornerRadius: number;
    /** Padding */
    padding: Insets;
}

/** TextDecorations - For use in RunStyle */
export interface TextDecorations {
    strikethrough: Line;
    underline: Line;
}

/** Line - Wraps an underlying NSUnderlineStyle. */
export interface Line {
    /** The color of the line. */
    color: Color;
    /** True enables the single flag. */
    single: boolean;
    /** True enables the thick flag. */
    thick: boolean;
    /** True enables the double flag. */
    double: boolean;
    /** True enables the patternSolid flag. */
    patternDot: boolean;
    /** True enables the patternDash flag. */
    patternDash: boolean;
    /** True enables the patternDashDot flag. */
    patternDashDot: boolean;
    /** True enables the patternDashDotDot flag. */
    patternDashDotDot: boolean;
    /** True enables the byWord flag. */
    byWord: boolean;
}

/** Insets - Wraps an underlying NSEdgeInsets. */
interface Insets {
    /** The top point value */
    top: number;
    /** The right point value */
    right: number;
    /** The bottom point value */
    bottom: number;
    /** The left point value */
    left: number;
}

interface InsetsConstructor {
    zero(): Insets;
    new (top: number, right: number, bottom: number, left: number): Insets;
}
  
export declare var Insets: InsetsConstructor;

/**
 * Font - Wraps an underlying FontDescriptor.
 *
 * This class can be a bit mysterious to work with. What is happening is you are
 * creating a description of a font that you want, but sometimes that fon't
 * might not exist. For example the following font might surprise:
 *
 * ```
 * new Font("Helvetica", 24).withMonospace()
 * ```
 *
 * You will likely get Helvetica at 24 points, but `withMonospaced` will have no
 * effect, because there isn't a monospaced version of Helvetica installed on
 * your computer.
 *
 * Generally if you are confused look into how NSFontDescriptor and
 * UIFontDescriptor work. This is a light wrapper around the macOS system font
 * descriptor.
 */
interface Font {

    /** The font family, ex. "Helvetica" */
    family: string;
    /** The font face, ex. "Bold" */
    face: string;
    /** The font size, ex. 12 */
    size: number;
    /** The font weight (-1 to 1) */
    weight: number;

    /** @returns A new font with italic trait */
    withItalics(): Font;
    
    /** @returns A new font with bold trait */
    withBold(): Font;
    
    /** @returns A new font with monospace trait */
    withMonospace(): Font;
    
    /** @returns A new font with small caps features */
    withSmallCaps(): Font;
    
    /** @returns A new font with lowercase small caps features */
    withLowercaseSmallCaps(): Font;
    
    /** @returns A new font with uppercase small caps features */
    withUppercaseSmallCaps(): Font;
    
    /** @returns A new font with monospaced digit features */
    withMonospacedDigit(): Font;
}

interface FontConstructor {

    systemBody(): Font;
    systemCallout(): Font;
    systemCaption1(): Font;
    systemCaption2(): Font;
    systemFootnote(): Font;
    systemHeadline(): Font;
    systemSubheadline(): Font;
    systemLargeTitle(): Font;
    systemTitle1(): Font;
    systemTitle2(): Font;
    systemTitle3(): Font;

    /**
     * @param name - The font family, ex. "Helvetica"
     * @param size - The font size, ex. 12
     */
    new (name: string, size: number): Font;
}

export declare var Font: FontConstructor;

/** Color - Wraps an unerlying CGColor. */
interface Color {

    /** Red component (0-1) */
    red: number;
    /** Green component (0-1) */
    green: number;
    /** Blue component (0-1) */
    blue: number;
    /** Alpha component (0-1) */
    alpha: number;

    /**
     * @param alpha - The alpha component (0-1)
     * @returns A new color with the same RGB components and the specified alpha component
     */
    withAlpha(alpha: number): Color;

    /**
     * @param fraction - The fraction of the color to blend with (0-1)
     * @param color - The color to blend with
     * @returns A new color with the specified fraction of the specified color blended in
     */
    withFraction(fraction: number, color: Color): Color;
}

interface ColorConstructor {
    black(): Color;
    white(): Color;
    clear(): Color;
    systemRed(): Color;
    systemGreen(): Color;
    systemBlue(): Color;
    systemOrange(): Color;
    systemYellow(): Color;
    systemBrown(): Color;
    systemPink(): Color;
    systemPurple(): Color;
    systemGray(): Color;
    systemTeal(): Color;
    systemIndigo(): Color;
    systemMint(): Color;
    systemCyan(): Color;

    /**
     * @param white - The white component (0-1)
     */
    gray(white: number): Color

    /**
     * @param red - The red component (0-1)
     * @param green - The green component (0-1)
     * @param blue - The blue component (0-1)
     * @param alpha - The alpha component (0-1)
     */
    new (red: number, green: number, blue: number, alpha: number): Color;
}

export declare var Color: ColorConstructor;
